'use strict';

var view = require('@codemirror/view');
var state = require('@codemirror/state');
var language = require('@codemirror/language');
var commands = require('@codemirror/commands');
var search = require('@codemirror/search');
var autocomplete = require('@codemirror/autocomplete');
var langMarkdown = require('@codemirror/lang-markdown');
var languageData = require('@codemirror/language-data');
var langJson = require('@codemirror/lang-json');
var highlight = require('@lezer/highlight');
var lezerJsonc = require('@yettoapp/lezer-jsonc');
var common = require('@lezer/common');
var lezerLiquid = require('@yettoapp/lezer-liquid');
var lint = require('@codemirror/lint');

function partialParse(ctx, parser, text, offset) {
    const innerTree = parser.parse(text);
    // Here we detach the syntax tree from the containing `Document` node, since
    // a parser expects that it needs to parse a full document. However, a child
    // `Document` element may or may not cause problems so we take it off its
    // root here.
    let treeElem = ctx.elt(innerTree, offset);
    const firstChild = innerTree.children[0];
    if (firstChild instanceof common.Tree) {
        treeElem = ctx.elt(firstChild, offset);
    }
    return treeElem;
}

const FRONTMATTER_START = '--{';
const FRONTMATTER_END = new RegExp(`^}--$`);
// Docs: https://github.com/lezer-parser/markdown#user-content-blockparser
const frontmatterParser = {
    // We need to give the parser a name
    name: 'Frontmatter',
    before: 'HorizontalRule',
    parse: (ctx, line) => {
        // This parser is inspired by the BlockParsers defined in
        // @lezer/markdown/src/markdown.ts
        if (line.text !== FRONTMATTER_START || ctx.lineStart !== 0) {
            return false;
        }
        // We have possible JSONC frontmatter. Now we need to look for the end of
        // the frontmatter.
        // Meanwhile, we'll be collecting all lines encountered so that we can parse
        // them into a JSONC AST.
        const jsoncLines = [];
        // We also need the position at which the (actual) frontmatter starts
        const from = 4;
        while (ctx.nextLine() && !FRONTMATTER_END.test(line.text)) {
            jsoncLines.push(line.text);
        }
        if (!FRONTMATTER_END.test(line.text)) {
            // The parser has collected the full rest of the document. This means
            // the frontmatter never stopped. In order to maintain readability, we
            // simply abort parsing.
            return false;
        }
        if (jsoncLines.length === 0) {
            return false; // Frontmatter must have content
        }
        // A final check: A frontmatter is NOT a valid document if there is
        // whitespace at the top (i.e. no blank lines between the delimiters and the
        // frontmatter content). NOTE: Whitespace AFTER the frontmatter content is
        // allowed!
        if (jsoncLines[0].trim() === '') {
            return false;
        }
        // At this point we have a full JSONC frontmatter; we know where
        // it starts and we know where it ends. In order to simplify creating the
        // required AST, we defer to letting the JSONC parser parse this thing into
        // a tree that we can then simply convert into the format consumed by
        // Codemirror. Keep in mind that we need to add the `{ }` back in, since
        // we've slurped them up as part of the frontmatter delimiters.
        const treeElem = partialParse(ctx, lezerJsonc.parser, `{${jsoncLines.join('\n')}`, from);
        const wrapperNode = ctx.elt('FencedCode', 0, ctx.lineStart + 3, [
            ctx.elt('JSONCFrontmatterStart', 0, 3),
            ctx.elt('JSONCFrontmatterMap', 4, ctx.lineStart - 1, [treeElem]),
            ctx.elt('JSONCFrontmatterEnd', ctx.lineStart, ctx.lineStart + 3)
        ]);
        ctx.addElement(wrapperNode);
        return true;
    }
};

const AsciiCurlyOpen = 123;
const AsciiPercent = 37;
const AsciiLiquidOpeners = new Set([AsciiCurlyOpen, AsciiPercent]);
const LiquidWrappers = /(?<opening>\{\{|\%)(?<code>.*)(?<closing>\%|\}\})/;
// https://github.com/lezer-parser/markdown#user-content-inlineparser
const liquidInlineParser = {
    name: 'inlineLiquid',
    parse: (ctx, next, pos) => {
        var _a, _b, _c;
        if (AsciiLiquidOpeners.has(next) && !AsciiLiquidOpeners.has(ctx.char(pos + 1)))
            return -1;
        const match = LiquidWrappers.exec(ctx.text);
        // Exit if no match
        if (!match)
            return -1;
        // Exit if there's no closing tag
        if (!((_a = match.groups) === null || _a === void 0 ? void 0 : _a.closing))
            return -1;
        // Exit if the match is full of empty space (e.g {{}} or {%     %}
        if (!((_c = (_b = match.groups) === null || _b === void 0 ? void 0 : _b.code) === null || _c === void 0 ? void 0 : _c.trim()))
            return -1;
        let curPos = pos;
        const lengths = {
            opening: match.groups.opening.length,
            code: match.groups.code.length,
            closing: match.groups.code.length
        };
        const childNodes = [ctx.elt('LiquidOutputStart', pos, lengths.opening)];
        curPos += lengths.opening;
        const treeElem = partialParse(ctx, lezerLiquid.parser, match.groups.code.trim(), lengths.opening);
        childNodes.push(treeElem);
        curPos += lengths.code;
        childNodes.push(ctx.elt('LiquidOutputEnd', curPos, curPos + lengths.closing));
        curPos += lengths.closing;
        const wrapperElem = ctx.elt('InlineCode', pos, curPos, childNodes);
        return ctx.addElement(wrapperElem);
    },
};

const customTags = {
    JSONCFrontmatterStart: highlight.Tag.define(highlight.tags.contentSeparator),
    JSONCFrontmatterEnd: highlight.Tag.define(highlight.tags.contentSeparator),
    JSONCFrontmatterMap: highlight.Tag.define(),
    LiquidOutputStart: highlight.Tag.define(highlight.tags.brace),
    LiquidOutputEnd: highlight.Tag.define(highlight.tags.brace),
    LiquidOutput: highlight.Tag.define()
};
function getMarkdownConfig() {
    return langMarkdown.markdown({
        base: langMarkdown.markdownLanguage,
        codeLanguages: languageData.languages,
        extensions: {
            parseBlock: [
                // This BlockParser parses JSONC frontmatter
                frontmatterParser
            ],
            parseInline: [
                liquidInlineParser
            ],
            // We have to notify the markdown parser about the additional Node Types
            // that the YAML block parser utilizes
            defineNodes: [
                { name: 'JSONCFrontmatterStart', style: customTags.JSONCFrontmatterStart, block: true },
                { name: 'JSONCFrontmatterEnd', style: customTags.JSONCFrontmatterEnd, block: true },
                { name: 'JSONCFrontmatterMap', style: customTags.JSONCFrontmatterMap, block: false },
                { name: 'LiquidOutputStart', style: customTags.LiquidOutputStart, block: false },
                { name: 'LiquidOutputEnd', style: customTags.LiquidOutputEnd, block: false },
                { name: 'LiquidOutput', style: customTags.LiquidOutput, block: false }
            ]
        }
    });
}
function getJsonConfig() {
    return langJson.json();
}
const LANGUAGE = new state.Compartment;

const config$1 = {
    name: 'yettoDark',
    dark: true,
    background: '#171717',
    foreground: '#fafafa',
    selection: '#3392FF44',
    cursor: '#c8e1ff',
    dropdownBackground: '#171717',
    dropdownBorder: '#1b1f23',
    activeLine: '#2b3036',
    matchingBracket: '#17E5E650',
    keyword: '#f97583',
    storage: '#f97583',
    variable: '#ffab70',
    parameter: '#e1e4e8',
    function: '#79b8ff',
    string: '#9ecbff',
    constant: '#79b8ff',
    type: '#79b8ff',
    class: '#b392f0',
    number: '#79b8ff',
    comment: '#6a737d',
    heading: '#79b8ff',
    invalid: '#f97583',
    regexp: '#9ecbff',
};
const yettoDarkTheme = view.EditorView.theme({
    '&': {
        color: config$1.foreground,
        backgroundColor: config$1.background,
    },
    '.cm-content': { caretColor: config$1.cursor },
    '.cm-cursor, .cm-dropCursor': { borderLeftColor: config$1.cursor },
    '&.cm-focused > .cm-scroller > .cm-selectionLayer .cm-selectionBackground, .cm-selectionBackground, .cm-content ::selection': { backgroundColor: config$1.selection },
    '.cm-panels': { backgroundColor: config$1.dropdownBackground, color: config$1.foreground },
    '.cm-panels.cm-panels-top': { borderBottom: '2px solid black' },
    '.cm-panels.cm-panels-bottom': { borderTop: '2px solid black' },
    '.cm-searchMatch': {
        backgroundColor: config$1.dropdownBackground,
        outline: `1px solid ${config$1.dropdownBorder}`
    },
    '.cm-searchMatch.cm-searchMatch-selected': {
        backgroundColor: config$1.selection
    },
    '.cm-activeLine': { backgroundColor: config$1.activeLine },
    '.cm-selectionMatch': { backgroundColor: config$1.selection },
    '&.cm-focused .cm-matchingBracket, &.cm-focused .cm-nonmatchingBracket': {
        backgroundColor: config$1.matchingBracket,
        outline: 'none'
    },
    '.cm-gutters': {
        backgroundColor: config$1.background,
        color: config$1.foreground,
        border: 'none'
    },
    '.cm-activeLineGutter': { backgroundColor: config$1.background },
    '.cm-foldPlaceholder': {
        backgroundColor: 'transparent',
        border: 'none',
        color: config$1.foreground
    },
    '.cm-tooltip': {
        border: `1px solid ${config$1.dropdownBorder}`,
        backgroundColor: config$1.dropdownBackground,
        color: config$1.foreground,
    },
    '.cm-tooltip .cm-tooltip-arrow:before': {
        borderTopColor: 'transparent',
        borderBottomColor: 'transparent'
    },
    '.cm-tooltip .cm-tooltip-arrow:after': {
        borderTopColor: config$1.foreground,
        borderBottomColor: config$1.foreground,
    },
    '.cm-tooltip-autocomplete': {
        '& > ul > li[aria-selected]': {
            background: config$1.selection,
            color: config$1.foreground,
        }
    }
}, { dark: config$1.dark });
const yettoDarkHighlightStyle = language.HighlightStyle.define([
    { tag: highlight.tags.keyword, color: config$1.keyword },
    { tag: [highlight.tags.name, highlight.tags.deleted, highlight.tags.character, highlight.tags.macroName], color: config$1.variable },
    { tag: [highlight.tags.propertyName], color: config$1.function },
    { tag: [highlight.tags.processingInstruction, highlight.tags.string, highlight.tags.inserted, highlight.tags.special(highlight.tags.string)], color: config$1.string },
    { tag: [highlight.tags.function(highlight.tags.variableName), highlight.tags.labelName], color: config$1.function },
    { tag: [highlight.tags.color, highlight.tags.constant(highlight.tags.name), highlight.tags.standard(highlight.tags.name)], color: config$1.constant },
    { tag: [highlight.tags.definition(highlight.tags.name), highlight.tags.separator], color: config$1.variable },
    { tag: [highlight.tags.className], color: config$1.class },
    { tag: [highlight.tags.number, highlight.tags.changed, highlight.tags.annotation, highlight.tags.modifier, highlight.tags.self, highlight.tags.namespace], color: config$1.number },
    { tag: [highlight.tags.typeName], color: config$1.type, fontStyle: config$1.type },
    { tag: [highlight.tags.operator, highlight.tags.operatorKeyword], color: config$1.keyword },
    { tag: [highlight.tags.url, highlight.tags.escape, highlight.tags.regexp, highlight.tags.link], color: config$1.regexp },
    { tag: [highlight.tags.meta, highlight.tags.comment], color: config$1.comment },
    { tag: highlight.tags.strong, fontWeight: 'bold' },
    { tag: highlight.tags.emphasis, fontStyle: 'italic' },
    { tag: highlight.tags.link, textDecoration: 'underline' },
    { tag: highlight.tags.heading, fontWeight: 'bold', color: config$1.heading },
    { tag: [highlight.tags.atom, highlight.tags.bool, highlight.tags.special(highlight.tags.variableName)], color: config$1.variable },
    { tag: highlight.tags.invalid, color: config$1.invalid },
    { tag: highlight.tags.strikethrough, textDecoration: 'line-through' },
]);
const yettoDark = [
    yettoDarkTheme,
    language.syntaxHighlighting(yettoDarkHighlightStyle),
];

const config = {
    name: 'yettoLight',
    dark: false,
    background: '#fafafa',
    foreground: '#171717',
    selection: '#0366d625',
    cursor: '#044289',
    dropdownBackground: '#fafafa',
    dropdownBorder: '#e1e4e8',
    activeLine: '#f6f8fa',
    matchingBracket: '#34d05840',
    keyword: '#d73a49',
    storage: '#d73a49',
    variable: '#e36209',
    parameter: '#24292e',
    function: '#005cc5',
    string: '#032f62',
    constant: '#005cc5',
    type: '#005cc5',
    class: '#6f42c1',
    number: '#005cc5',
    comment: '#6a737d',
    heading: '#005cc5',
    invalid: '#cb2431',
    regexp: '#032f62',
};
const yettoLightTheme = view.EditorView.theme({
    '&': {
        color: config.foreground,
        backgroundColor: config.background,
    },
    '.cm-content': { caretColor: config.cursor },
    '.cm-cursor, .cm-dropCursor': { borderLeftColor: config.cursor },
    '&.cm-focused > .cm-scroller > .cm-selectionLayer .cm-selectionBackground, .cm-selectionBackground, .cm-content ::selection': { backgroundColor: config.selection },
    '.cm-panels': { backgroundColor: config.dropdownBackground, color: config.foreground },
    '.cm-panels.cm-panels-top': { borderBottom: '2px solid black' },
    '.cm-panels.cm-panels-bottom': { borderTop: '2px solid black' },
    '.cm-searchMatch': {
        backgroundColor: config.dropdownBackground,
        outline: `1px solid ${config.dropdownBorder}`
    },
    '.cm-searchMatch.cm-searchMatch-selected': {
        backgroundColor: config.selection
    },
    '.cm-activeLine': { backgroundColor: config.activeLine },
    '.cm-selectionMatch': { backgroundColor: config.selection },
    '&.cm-focused .cm-matchingBracket, &.cm-focused .cm-nonmatchingBracket': {
        backgroundColor: config.matchingBracket,
        outline: 'none'
    },
    '.cm-gutters': {
        backgroundColor: config.background,
        color: config.foreground,
        border: 'none'
    },
    '.cm-activeLineGutter': { backgroundColor: config.background },
    '.cm-foldPlaceholder': {
        backgroundColor: 'transparent',
        border: 'none',
        color: config.foreground
    },
    '.cm-tooltip': {
        border: `1px solid ${config.dropdownBorder}`,
        backgroundColor: config.dropdownBackground,
        color: config.foreground,
    },
    '.cm-tooltip .cm-tooltip-arrow:before': {
        borderTopColor: 'transparent',
        borderBottomColor: 'transparent'
    },
    '.cm-tooltip .cm-tooltip-arrow:after': {
        borderTopColor: config.foreground,
        borderBottomColor: config.foreground,
    },
    '.cm-tooltip-autocomplete': {
        '& > ul > li[aria-selected]': {
            background: config.selection,
            color: config.foreground,
        }
    }
}, { dark: config.dark });
const yettoLightHighlightStyle = language.HighlightStyle.define([
    { tag: highlight.tags.keyword, color: config.keyword },
    { tag: [highlight.tags.name, highlight.tags.deleted, highlight.tags.character, highlight.tags.macroName], color: config.variable },
    { tag: [highlight.tags.propertyName], color: config.function },
    { tag: [highlight.tags.processingInstruction, highlight.tags.string, highlight.tags.inserted, highlight.tags.special(highlight.tags.string)], color: config.string },
    { tag: [highlight.tags.function(highlight.tags.variableName), highlight.tags.labelName], color: config.function },
    { tag: [highlight.tags.color, highlight.tags.constant(highlight.tags.name), highlight.tags.standard(highlight.tags.name)], color: config.constant },
    { tag: [highlight.tags.definition(highlight.tags.name), highlight.tags.separator], color: config.variable },
    { tag: [highlight.tags.className], color: config.class },
    { tag: [highlight.tags.number, highlight.tags.changed, highlight.tags.annotation, highlight.tags.modifier, highlight.tags.self, highlight.tags.namespace], color: config.number },
    { tag: [highlight.tags.typeName], color: config.type, fontStyle: config.type },
    { tag: [highlight.tags.operator, highlight.tags.operatorKeyword], color: config.keyword },
    { tag: [highlight.tags.url, highlight.tags.escape, highlight.tags.regexp, highlight.tags.link], color: config.regexp },
    { tag: [highlight.tags.meta, highlight.tags.comment], color: config.comment },
    { tag: highlight.tags.strong, fontWeight: 'bold' },
    { tag: highlight.tags.emphasis, fontStyle: 'italic' },
    { tag: highlight.tags.link, textDecoration: 'underline' },
    { tag: highlight.tags.heading, fontWeight: 'bold', color: config.heading },
    { tag: [highlight.tags.atom, highlight.tags.bool, highlight.tags.special(highlight.tags.variableName)], color: config.variable },
    { tag: highlight.tags.invalid, color: config.invalid },
    { tag: highlight.tags.strikethrough, textDecoration: 'line-through' },
]);
const yettoLight = [
    yettoLightTheme,
    language.syntaxHighlighting(yettoLightHighlightStyle),
];

const THEME = new state.Compartment;
const toggleTheme = (view) => {
    const newTheme = THEME.get(view.state) == yettoLight ? yettoDark : yettoLight;
    view.dispatch({
        effects: [
            THEME.reconfigure(newTheme),
        ]
    });
    return true;
};

function prependLines(mark, view, ordered = false) {
    view.dispatch(view.state.changeByRange(range => {
        const changes = [];
        let firstLine = view.state.doc.lineAt(range.from).number;
        let lastLine = view.state.doc.lineAt(range.to).number;
        let listIdx = 1;
        let newHead = range.to;
        // Iterate over the lines
        for (let i = firstLine; i <= lastLine; i++, listIdx++) {
            let line = view.state.doc.line(i);
            let text = ordered ? `${listIdx}${mark} ` : `${mark} `;
            newHead += text.length;
            // Create a change that inserts the string at the start of the line
            changes.push({
                from: line.from,
                insert: text
            });
        }
        // TODO: Figure out how to get the head value correct after prepending
        return {
            range: state.EditorSelection.range(range.from, newHead),
            changes
        };
    }));
}
function makePrependLinesCommand(mark, ordered = false) {
    return (view) => {
        prependLines(mark, view, ordered);
        return true;
    };
}
const BulletedList = {
    key: "Ctrl-Shift-8",
    mac: "Cmd-Shift-8",
    run: makePrependLinesCommand("-"),
    preventDefault: true
};
const NumberedList = {
    key: "Ctrl-Shift-7",
    mac: "Cmd-Shift-7",
    run: makePrependLinesCommand(".", true),
    preventDefault: true
};
const TaskList = {
    key: "Ctrl-Shift-l",
    mac: "Meta-Shift-l",
    run: makePrependLinesCommand("- [ ]"),
    preventDefault: true
};
const QuoteText = {
    key: "Ctrl-Shift-.",
    mac: "Cmd-Shift-.",
    run: makePrependLinesCommand(">"),
    preventDefault: true
};

function wrapText(mark, view) {
    let changes = view.state.changeByRange(range => {
        // Get the current selection
        let { from, to } = range;
        // Get the text surrounding the selection to check for bold syntax
        const before = view.state.sliceDoc(Math.max(0, from - mark.length), from);
        const after = view.state.sliceDoc(to, Math.min(to + mark.length, view.state.doc.length));
        let tr;
        // Check if the selected text is already wrapped in the marker
        if (before === mark && after === mark) {
            // If it is, remove the formatting
            tr = {
                changes: [
                    { from: from - mark.length, to: from },
                    { from: to, to: to + mark.length }
                ],
                range: state.EditorSelection.range(from - mark.length, to + mark.length)
            };
        }
        else {
            // If it's not, add the formatting
            tr = {
                changes: [
                    { from, to: from, insert: mark },
                    { from: to, to, insert: mark }
                ],
                range: state.EditorSelection.range(from + mark.length, to + mark.length)
            };
        }
        return tr;
    });
    view.dispatch(changes);
}
function makeWrapTextCommand(mark) {
    return (view) => {
        wrapText(mark, view);
        return true;
    };
}
const BoldText = {
    key: "Ctrl-b",
    mac: "Cmd-b",
    run: makeWrapTextCommand("**"),
    preventDefault: true,
};
const ItalicText = {
    key: "Ctrl-i",
    mac: "Cmd-i",
    run: makeWrapTextCommand("*"),
    preventDefault: true,
};
const CodeText = {
    key: "Ctrl-e",
    mac: "Cmd-e",
    run: makeWrapTextCommand("`"),
    preventDefault: true,
};

const options = [
    {
        label: "/image",
        type: "text",
        apply: autocomplete.snippet("![${1:alt_text}](${2:url})\n\n${3}"),
        detail: "An inline image",
        section: "Markdown"
    },
    {
        label: "/link",
        type: "text",
        apply: autocomplete.snippet("[${1:link_text}](${2:url}) ${3}"),
        detail: "A link to some other place",
        section: "Markdown"
    },
    {
        label: "/codeBlock",
        type: "text",
        apply: autocomplete.snippet("```${1:language}\n${2:code}\n```"),
        detail: "A block of code or other preformatted text",
        section: "Markdown"
    },
];
function markdownCompletions(context) {
    let word = context.matchBefore(/\/\w*/);
    if (!word)
        return;
    if (word.from == word.to && !context.explicit)
        return null;
    return {
        from: word.from,
        options,
    };
}

class ImageWidget extends view.WidgetType {
    constructor({ url, classes }) {
        super();
        this.url = url;
        this.classes = classes;
    }
    eq(imageWidget) {
        return imageWidget.url === this.url;
    }
    toDOM() {
        var _a, _b;
        const container = document.createElement('div');
        const image = container.appendChild(document.createElement('img'));
        container.setAttribute('aria-hidden', 'true');
        container.className = ((_a = this.classes) === null || _a === void 0 ? void 0 : _a.container) || "";
        image.className = ((_b = this.classes) === null || _b === void 0 ? void 0 : _b.img) || "";
        image.src = this.url;
        return container;
    }
}
const images = (styles = null) => {
    const imageRegex = /!\[.*?\]\((?<url>.*?\.(png|jpeg|jpg|gif|ico))\)/;
    const imageDecoration = (imageWidgetParams) => view.Decoration.widget({
        widget: new ImageWidget(imageWidgetParams),
        side: 1,
        block: true,
    });
    const decorate = (state$1) => {
        const widgets = [];
        language.syntaxTree(state$1).iterate({
            enter: ({ type, from, to }) => {
                var _a;
                if (type.name === 'Image') {
                    const result = imageRegex.exec(state$1.doc.sliceString(from, to));
                    if ((_a = result === null || result === void 0 ? void 0 : result.groups) === null || _a === void 0 ? void 0 : _a.url) {
                        const widgetParams = Object.assign({ url: result.groups.url }, styles && { classes: styles });
                        widgets.push(imageDecoration(widgetParams).range(state$1.doc.lineAt(to).to));
                    }
                }
            },
        });
        return widgets.length > 0 ? state.RangeSet.of(widgets) : view.Decoration.none;
    };
    const imagesField = state.StateField.define({
        create(state) {
            return decorate(state);
        },
        update(images, transaction) {
            if (transaction.docChanged)
                return decorate(transaction.state);
            return images.map(transaction.changes);
        },
        provide(field) {
            return view.EditorView.decorations.from(field);
        },
    });
    return [
        imagesField,
    ];
};

const KEYBINDINGS = new state.Compartment;
const keymaps = () => {
    return view.keymap.of([
        NumberedList,
        BulletedList,
        TaskList,
        QuoteText,
        BoldText,
        ItalicText,
        CodeText,
        commands.indentWithTab,
        ...commands.defaultKeymap,
        ...lint.lintKeymap,
        ...autocomplete.completionKeymap,
        ...commands.historyKeymap,
        ...search.searchKeymap,
        ...autocomplete.closeBracketsKeymap,
    ]);
};

function getSthenoConfig(lang, ...extensions) {
    const language$1 = lang === 'markdown' ? getMarkdownConfig() : getJsonConfig();
    const config = {
        extensions: [
            view.highlightSpecialChars(),
            commands.history(),
            view.drawSelection(),
            view.dropCursor(),
            view.EditorView.lineWrapping,
            state.EditorState.allowMultipleSelections.of(true),
            autocomplete.autocompletion(),
            search.highlightSelectionMatches(),
            language.indentOnInput(),
            language.bracketMatching(),
            autocomplete.closeBrackets(),
            KEYBINDINGS.of(keymaps()),
            THEME.of(yettoLight),
            LANGUAGE.of(language$1),
            language$1.language.data.of({
                autocomplete: markdownCompletions
            }),
            ...extensions,
        ],
    };
    return config;
}

exports.BulletedList = BulletedList;
exports.NumberedList = NumberedList;
exports.QuoteText = QuoteText;
exports.THEME = THEME;
exports.TaskList = TaskList;
exports.getSthenoConfig = getSthenoConfig;
exports.images = images;
exports.makePrependLinesCommand = makePrependLinesCommand;
exports.makeWrapTextCommand = makeWrapTextCommand;
exports.prependLines = prependLines;
exports.toggleTheme = toggleTheme;
exports.wrapText = wrapText;
exports.yettoDark = yettoDark;
exports.yettoLight = yettoLight;
